/*
 * Barebones code to turn on backlight and panel power on PCB10006.
 *
 */


#include "derivative.h" /* include peripheral declarations */

/* 
 * Define color mix vals for possible operation states.
 * Tracked states are Screen on, Screen off, No signal (standby)
 * Possible values: [0..255]
 */
#define LED_STBY_RED 255
#define LED_STBY_GREEN 128
#define LED_STBY_BLUE 0
#define LED_ON_RED 255
#define LED_ON_GREEN 255
#define LED_ON_BLUE 255
#define LED_OFF_RED 32
#define LED_OFF_GREEN 0
#define LED_OFF_BLUE 0

void setLEDColor(unsigned char red, unsigned char green, unsigned char blue);

#define RED_LED_LOC		(1<<2)
#define GREEN_LED_LOC	(1<<3)
#define BLUE_LED_LOC	(1<<4)
#define PNLPWR_ENA_LOC  (1<<7)
#define BL_ENA_LOC      (1<<5)
#define BL_PWM_LOC      (1<<4)

#define RED_LED_OFF  GPIOC_PSOR = RED_LED_LOC
#define RED_LED_ON   GPIOC_PCOR = RED_LED_LOC

#define GREEN_LED_OFF  GPIOC_PSOR = GREEN_LED_LOC
#define GREEN_LED_ON   GPIOC_PCOR = GREEN_LED_LOC

#define BLUE_LED_OFF  GPIOC_PSOR = BLUE_LED_LOC
#define BLUE_LED_ON   GPIOC_PCOR = BLUE_LED_LOC

#define PNLPWR_ENA_ON GPIOD_PSOR = PNLPWR_ENA_LOC
#define PNLPWR_ENA_OFF GPIOD_PCOR = PNLPWR_ENA_LOC

#define BL_ENA_ON GPIOD_PSOR = BL_ENA_LOC
#define BL_ENA_OFF GPIOD_PCOR = BL_ENA_LOC

#define BL_PWM_ON GPIOD_PSOR = BL_PWM_LOC
#define BL_PWM_OFF GPIOD_PCOR = BL_PWM_LOC


#define TPM_C 48e6                        // core clock (only for calculation, I does not afect real clock)
#define PWM_FREQ 16000                    // PWM frequency [Hz]
#define MODULO ( TPM_C / PWM_FREQ ) // calculation the modulo for FTM0 


void Delay(int Ticks)
{
	int i;
	
	for(i=0;i<Ticks;i++)
	{
	}
	
}
int main(void)
{
	SIM_COPC = 0; //Disable watchdog
	SIM_CLKDIV1 = 0; //MCLKDIV/1, BUSCLKDIV/1 (defaults)
	SIM_SOPT2 &= ~SIM_SOPT2_PLLFLLSEL_MASK; //Disable MCGPLLCLK/2 (default)
	SIM_SOPT1 |= SIM_SOPT1_OSC32KSEL(3); //Set 32K CLK = LPO 1KHz Clk
	SIM_SOPT2 = (SIM_SOPT2 & ~SIM_SOPT2_TPMSRC(2)) | SIM_SOPT2_TPMSRC(1); //Set TPM Cksrc=MCGFLLCLK
	
	MCG_C1 = (MCG_C1_IREFS_MASK | MCG_C1_IRCLKEN_MASK); //Select internal refclk, int refclk enable
	MCG_C2 = 0; //Generate interrupt request on loss of ext refclk
	MCG_C4 &= ~(MCG_C4_DMX32_MASK | MCG_C4_DRST_DRS(3)); //Set FLL factor 640, loose 32K ref range
	OSC0_CR = OSC_CR_ERCLKEN_MASK; //Enable ext ref clk
	MCG_C5 = 0; //(default)
	MCG_C6 = 0; //(default)
	while((MCG_S & MCG_S_IREFST_MASK) == 0); //Wait until FLL refclk = int refclk
	while((MCG_S & MCG_S_CLKST(3)) != 0); //Wait until clock = FLLclk

	SIM_SCGC5 |= SIM_SCGC5_PORTC_MASK;
	SIM_SCGC6 |= SIM_SCGC6_TPM0_MASK;
	
	TPM0_SC = 0;
	TPM0_CNT = 0;
	TPM0_C0SC = 0;
	TPM0_C1SC = 0;
	TPM0_C2SC = 0;
	TPM0_C3SC = 0;
	TPM0_C4SC = 0;
	TPM0_C5SC = 0;
	TPM0_MOD = TPM_MOD_MOD(0x9FFF);
	TPM0_C1SC = TPM_CnSC_MSB_MASK | TPM_CnSC_ELSB_MASK | TPM_CnSC_ELSA_MASK;
	TPM0_C1V = TPM_CnV_VAL(0x00A4);
	TPM0_C2SC = TPM_CnSC_MSB_MASK | TPM_CnSC_ELSB_MASK | TPM_CnSC_ELSA_MASK;
	TPM0_C2V = TPM_CnV_VAL(0x00A4);
	TPM0_C3SC = TPM_CnSC_MSB_MASK | TPM_CnSC_ELSB_MASK | TPM_CnSC_ELSA_MASK;
	TPM0_C3V = TPM_CnV_VAL(0x00A4);
	
	PORTC_PCR2 = PORT_PCR_MUX(4) | PORT_PCR_DSE_MASK;
	PORTC_PCR3 = PORT_PCR_MUX(4) | PORT_PCR_DSE_MASK;
	PORTC_PCR4 = PORT_PCR_MUX(4) | PORT_PCR_DSE_MASK;
	
	TPM0_SC = TPM_SC_CMOD(1) | TPM_SC_PS(0); //Inc on LPTPM clk, Div/1
	
	SIM_SCGC5 |= SIM_SCGC5_PORTD_MASK;
	PORTD_PCR4 = PORT_PCR_MUX(1) | PORT_PCR_DSE_MASK;
	PORTD_PCR5 = PORT_PCR_MUX(1) | PORT_PCR_DSE_MASK;
	PORTD_PCR7 = PORT_PCR_MUX(1) | PORT_PCR_DSE_MASK;
	GPIOD_PDDR = PNLPWR_ENA_LOC | BL_ENA_LOC | BL_PWM_LOC;
	PNLPWR_ENA_ON;
	BL_ENA_ON;
	BL_PWM_ON;

	while(1);

	return 0;
}

void setLEDColor(unsigned char red, unsigned char green, unsigned char blue)
{
	//doStuff;
}

